import { IMessageHeader, ErrorCodes } from "./types";
import { Serializeable } from "./serializeable";

export class Header implements IMessageHeader, Serializeable {
   ID: number;
   QR: 0 | 1;
   OPCODE: number
   AA: 0 | 1;
   TC: 0 | 1;
   RD: 0 | 1;
   RA: 0 | 1;
   Z: 0 | 1;
   AD: 0 | 1;
   CD: 0 | 1;
   RCODE: ErrorCodes;
   QDCOUNT: number;
   ANCOUNT: number;
   NSCOUNT: number;
   ARCOUNT: number;

   constructor(header: IMessageHeader) {
      for (let k in header) {
         this[k] = header[k];
      }
   }

   lock() {
      var f = 0x0000;
      f = f | (this.QR << 15);
      f = f | (this.OPCODE << 11);
      f = f | (this.AA << 10);
      f = f | (this.TC << 9);
      f = f | (this.RD << 8);
      f = f | (this.RA << 7);
      f = f | (this.Z << 6);
      f = f | (this.AD << 5);
      f = f | (this.CD << 4);
      f = f | this.RCODE;

      let id = this.ID;
      let qdc = this.QDCOUNT;
      let anc = this.ANCOUNT;
      let nsc = this.NSCOUNT;
      let arc = this.ARCOUNT;
      return {
         length: 12,
         serialize: (buffer: Buffer, offset: number) => {
            buffer.writeUInt16BE(id, offset);
            offset += 2;
            buffer.writeUInt16BE(f, offset);
            offset += 2;
            buffer.writeUInt16BE(qdc, offset);
            offset += 2;
            buffer.writeUInt16BE(anc, offset);
            offset += 2;
            buffer.writeUInt16BE(nsc, offset);
            offset += 2;
            buffer.writeUInt16BE(arc, offset);
            return offset + 2;
         }
      }
   }

   // serialize() {
   //    let data = Buffer.alloc(12);
   //    data.writeUInt16BE(this.ID, 0);

   //    data.writeUInt16BE(f, 2);
   //    data.writeUInt16BE(this.QDCOUNT, 4)
   //    data.writeUInt16BE(this.ANCOUNT, 6)
   //    data.writeUInt16BE(this.NSCOUNT, 8)
   //    data.writeUInt16BE(this.ARCOUNT, 10)
   //    return data;
   // }
}